unit BrowserMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OleCtrls, SHDocVw, ComCtrls, ExtCtrls, StdCtrls, ToolWin, ActnList, Menus,
  ImgList, StdActns;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    WebBrowser1: TWebBrowser;
    HtmlRichEdit: TRichEdit;
    OpenDialog1: TOpenDialog;
    ToolBar1: TToolBar;
    MainMenu1: TMainMenu;
    ActionList1: TActionList;
    ToolButton1: TToolButton;
    Open1: TAction;
    Exit1: TAction;
    File1: TMenuItem;
    Open2: TMenuItem;
    N1: TMenuItem;
    Exit2: TMenuItem;
    ToolbarImages: TImageList;
    ToolButton2: TToolButton;
    EditCopy1: TEditCopy;
    EditCut1: TEditCut;
    EditDelete1: TEditDelete;
    EditPaste1: TEditPaste;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    Edit1: TMenuItem;
    Copy1: TMenuItem;
    Cut1: TMenuItem;
    Delete1: TMenuItem;
    Paste1: TMenuItem;
    EditUndo1: TEditUndo;
    ToolButton7: TToolButton;
    Undo1: TMenuItem;
    FontDialog1: TFontDialog;
    Font1: TAction;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    Options1: TMenuItem;
    Font2: TMenuItem;
    WordWrap1: TAction;
    Wordwrap2: TMenuItem;
    ToolButton10: TToolButton;
    procedure WebBrowser1NavigateComplete2(Sender: TObject;
      const pDisp: IDispatch; var URL: OleVariant);
    procedure PageControl1Change(Sender: TObject);
    procedure Open1Execute(Sender: TObject);
    procedure Exit1Execute(Sender: TObject);
    procedure Font1Execute(Sender: TObject);
    procedure Font1Update(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure WordWrap1Execute(Sender: TObject);
  private
    FHtmlCache: string;
  public
    procedure OpenHTML(const HTML: string);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

{ TMainForm }

procedure TMainForm.OpenHTML(const HTML: string);
var
  Document: Variant;
begin
  if WebBrowser1.Document = nil then
  begin
    FHtmlCache := HTML;
    WebBrowser1.Navigate('about:blank');
  end else
  begin
    FHtmlCache := '';
    Document := WebBrowser1.Document;
    Document.close;
    Document.write(HTML);
  end;
end;

procedure TMainForm.WebBrowser1NavigateComplete2(Sender: TObject;
  const pDisp: IDispatch; var URL: OleVariant);
var
  Document: Variant;
begin
  if FHtmlCache <> '' then
  begin
    Document := TWebBrowser(Sender).Document;
    Document.write(FHtmlCache);
    FHtmlCache := '';
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  PageControl1.ActivePageIndex := 1;
end;

procedure TMainForm.PageControl1Change(Sender: TObject);
begin
  if PageControl1.ActivePageIndex = 0 then
    OpenHTML(HtmlRichEdit.Text);
end;

procedure TMainForm.Open1Execute(Sender: TObject);
begin
  with OpenDialog1 do
  begin
    FileName := '';
    if Execute then
    begin
      PageControl1.ActivePageIndex := 1;
      HtmlRichEdit.Lines.LoadFromFile(FileName);
    end;
  end;
end;

procedure TMainForm.Exit1Execute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.Font1Execute(Sender: TObject);
begin
  with FontDialog1 do
  begin
    Font := HtmlRichEdit.Font;
    if Execute then
      HtmlRichEdit.Font := Font;
  end;    
end;

procedure TMainForm.Font1Update(Sender: TObject);
begin
  TAction(Sender).Enabled := (PageControl1.ActivePageIndex = 1); 
end;

procedure TMainForm.WordWrap1Execute(Sender: TObject);
var
  SelSave: Integer;
begin
  with WordWrap1 do
  begin
    Checked := not Checked;
    SelSave := HtmlRichEdit.SelStart;
    HtmlRichEdit.WordWrap := Checked;
    HtmlRichEdit.SelStart := SelSave;
    SendMessage(HtmlRichEdit.Handle, EM_SCROLLCARET, 0, 0);
  end;
end;

end.
