{*************************************************************}
{               Korzh's classes and procedures                }
{                   Copyright  Korzh.com                     }
{                http://www.korzh.com/delphi/                 }
{                  mailto:delphi@korzh.com                    }
{-------------------------------------------------------------}
{             Date/Time functions and procedures              }
{*************************************************************}
unit KDattime;

interface
uses Classes, SysUtils;

//returns string that represent the date passed in ADate parameter
//Jan-1-2000 -> '1 of January 2000'
function GetDateName(ADate : TDateTime) : string;

//returns first date of the year by some date in this year
function GetFirstYearDate(ADate : TDateTime) : TDateTime;

//returns first fate of quarter by year and quarter number
function GetFirstQuarterDate(AYear, QNum : word) : TDateTime;

//returns first fate of month by some date in this month
function GetFirstMonthDate(ADate : TDateTime) : TDateTime;

//returns first fate of next month by year and number of month
function NextMonthFirstDay(AYear, AMonth : word) : TDateTime;

//returns last date in the year by some date in this year
function GetLastYearDate(ADate : TDateTime) : TDateTime;

//returns last date in the qurter by year and quarter number
function GetLastQuarterDate(AYear, QNum : word) : TDateTime;

//returns last date of the month by some date in this month
function GetLastMonthDate(ADate : TDateTime) : TDateTime;

//returns last date of the month by year and month number
function LastMonthDate(AYear, AMonth : integer) : TDateTime;

//returns number of quarter by date
function GetQuartal(ADate : TDateTime) : integer;

//returns number of a quarter as Roman numerals
function GetQuartalWrite(AQuartal : word) : string;

//get number of the year by date
function GetYear(ADate : TDateTime) : word;

//get number of the month by date
function GetMonth(ADate : TDateTime) : word;

//get number of the day by date
function GetDay(ADate : TDateTime) : word;

//add PlusYear years, PlusMonth months and PlusDay days
//to the date passed in ADate parameter and returns the result
function AddDate(ADate : TDateTime; PlusYear, PlusMonth, PlusDay : word) : TDateTime;

//add PlusHour hours, PlusMin minutes and PlusSec seconds
//to the time passed in ATime parameter and returns the result
function AddTime(ATime : TDateTime; PlusHour, PlusMin, PlusSec : word) : TDateTime;

//fill AMonths list by names of all months
procedure GetMonthNames(AMonths : TStrings);

//initilize global variables ShortYearName and LongMonthNames2
procedure InitDateTimeParams(const AShortYearName : string; ALongMonthNames2 : array of const);

function StrToDateTimeByFormat(const FormatStr, DateStr : string) : TDateTime;

var
  //contains names of month for GetDateName function
  //e.g. LongMonthNames2[1] can be 'of January'
  LongMonthNames2 : array [1..12] of string;

  //represents name of the year
  //(empty for English but can be useful for other languages
  //e.g. '2000 p.' in Russian)
  ShortYearName : string = '';

implementation
uses KStrings;

function GetDateName(ADate : TDateTime) : string;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := IntToStr(WDay) + ' ' + LongMonthNames2[WMonth] + ' ' + IntToStr(WYear) + ShortYearName
end;

function GetFirstYearDate(ADate : TDateTime) : TDateTime;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := EncodeDate(WYear, 1, 1);
end;

function GetLastYearDate(ADate : TDateTime) : TDateTime;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := EncodeDate(WYear, 12, 31);
end;

function GetFirstMonthDate(ADate : TDateTime) : TDateTime;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := EncodeDate(WYear, WMonth, 1);
end;

function GetLastMonthDate(ADate : TDateTime) : TDateTime;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  if WMonth = 12 then
  begin
    WMonth := 1;
    inc(WYear)
  end
  else
    inc(WMonth);
  Result := EncodeDate(WYear, WMonth, 1) - 1;
end;

function NextMonthFirstDay(AYear, AMonth : word) : TDateTime;
begin
  if AMonth = 12 then
  begin
    AMonth := 1;
    inc(AYear);
  end
  else
    inc(AMonth);
  Result := EncodeDate(AYear, AMonth, 1);
end;

function GetFirstQuarterDate(AYear, QNum : word) : TDateTime;
begin
  Result := EncodeDate(AYear, QNum * 3, 1);
end;

function GetLastQuarterDate(AYear, QNum : word) : TDateTime;
begin
  Result := LastMonthDate(AYear, QNum * 3);
end;


function LastMonthDate(AYear, AMonth : integer) : TDateTime;
begin
  if AMonth = 12 then
  begin
    AMonth := 1;
    inc(AYear)
  end
  else
    inc(AMonth);
  Result := EncodeDate(AYear, AMonth, 1) - 1;
end;

function GetQuartal(ADate : TDateTime) : integer;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := ((WMonth - 1) div 3) + 1;
end;

function GetQuartalWrite(AQuartal : word) : string;
const
  Quartals : array[1..4] of string=('I','II','III','IV');
begin
  Result := Quartals[AQuartal];
end;

function GetYear(ADate : TDateTime) : word;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := WYear;
end;

function GetMonth(ADate : TDateTime) : word;
var
  WDay, WMonth, WYear : word;
begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := WMonth;
end;

function GetDay(ADate : TDateTime) : word;
var
  WDay, WMonth, WYear : word;
Begin
  DecodeDate(ADate, WYear, WMonth, WDay);
  Result := WDay;
end;

function AddDate(ADate : TDateTime; PlusYear, PlusMonth, PlusDay : word) : TDateTime;
var
  WYear, WMonth, WDay : word;
begin
  Result := ADate + PlusDay;

  DecodeDate(Result, WYear, WMonth, WDay);
  inc(WMonth, PlusMonth);
  if WMonth > 12 then
  begin
    inc(PlusYear, (WMonth - 1) div 12);
    WMonth := (WMonth - 1) mod 12 + 1;
  end;

  inc(WYear, PlusYear);

  Result := EncodeDate(WYear, WMonth, WDay);
end;

function AddTime(ATime : TDateTime; PlusHour, PlusMin, PlusSec : word) : TDateTime;
var
  WHour, WMin, WSec, WMsec : word;
  PlusDay : word;
begin
  DecodeTime(ATime, WHour, WMin, WSec, WMSec);

  PlusDay := 0;
  inc(WSec, PlusSec);

  if WSec > 59 then
  begin
    inc(WMin, WSec div 60);
    WSec := WSec mod 60;
  end;

  inc(WMin, PlusMin);
  if WMin > 59 then
  begin
    inc(WHour, WMin div 60);
    WMin := WMin mod 60;
  end;

  inc(WHour, PlusHour);
  if WHour > 23 then
  begin
    inc(PlusDay, WHour div 24);
    WHour := WHour mod 24;
  end;

  Result := int(ATime) + PlusDay + EncodeTime(WHour, WMin, WSec, WMSec);
end;

procedure GetMonthNames(AMonths : TStrings);
var
  i : integer;
begin
 AMonths.Clear;
 for i := 1 to 12 do AMonths.Add(LongMonthNames[i]);
end;

procedure DateTimeInit;
var
  I : integer;
begin
  for I := 1 to 12 do
    LongMonthNames2[i] := 'of ' + LongMonthNames[I];
end;

procedure InitDateTimeParams(const AShortYearName : string; ALongMonthNames2 : array of const);
var
  I : integer;
begin
  ShortYearName := AShortYearName;
  for I := 0 to High(ALongMonthNames2) do
    LongMonthNames2[I + 1] := string(ALongMonthNames2[I].VAnsiString);
end;

function StrToDateTimeByFormat(const FormatStr, DateStr : string) : TDateTime;
var
  PrevFormat : string;
  PrevDateSeparator : char;
  SeparatorStr : string;
  P, LP : integer;
begin
  PrevFormat := SysUtils.ShortDateFormat;
  SysUtils.ShortDateFormat := FormatStr;

  PrevDateSeparator := SysUtils.DateSeparator;

  P := 1;
  while LowerCase(FormatStr[P])[1] in ['d','m','y'] do inc(P);
  LP := P;
  while not (LowerCase(FormatStr[P])[1] in ['d','m','y']) do inc(P);
  SeparatorStr := Copy(FormatStr, LP, P - LP);

  if SeparatorStr[1] = '"' then
    SysUtils.DateSeparator := UnquotedString(SeparatorStr, '"')[1]
  else if SeparatorStr <> '/' then
    SysUtils.DateSeparator := SeparatorStr[1];

  Result := StrToDateTime(DateStr);
  SysUtils.ShortDateFormat := PrevFormat;
  SysUtils.DateSeparator := PrevDateSeparator;
end;


initialization
  DateTimeInit;
end.
