{Reading a file to a string
Perhaps the easiest way to read a file into a string is using the LoadFromFile method of a TStringList object and then accessing its Text property:}
uses SysUtils, Classes;

function LoadFile(const FileName: TFileName): string;
var
  StringList: TStringList;
begin
  StringList := TStringList.Create;
  try
    StringList.LoadFromFile(FileName);
    Result := StringList.Text;
  except
    StringList.Free;
    raise;
  end;
  StringList.Free;
end;
However, this way of doing things is inefficient, since LoadFromFile loads the file and parses it to separate the lines, and then when we use the Text property, an internal method is called to produce a string joining the lines, so this double job is done for nothing, plus we end up using more than the double of the memory storage we actually need (until we free the TStringList object).

--------------------------------------------------------------------------------

{A better way
A better approach could be using a TFileStream object (or the old Assign, Reset, Read and Close procedures) to directly read the contents of a file into a string. Here is the same function using TFileStream that returns the content of the file whose name is passed as parameter:}
uses SysUtils, Classes;

function LoadFile(const FileName: TFileName): string;
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName,
    fmOpenRead or fmShareDenyWrite);
  try
    SetLength(Result, Stream.Size);
    Stream.Read(Pointer(Result)^, Stream.Size);
  except
    Result := '';  // Deallocates memory
    Stream.Free;
    raise;
  end;
  Stream.Free;
end;


--------------------------------------------------------------------------------

{Writing a file from a string
If we needed to save the contents of string back to disk, we can use the following procedure:}
uses SysUtils, Classes;

procedure SaveFile(const FileName: TFileName;
                   const content: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    Stream.Write(Pointer(content)^, Length(content));
  except
    Stream.Free;
    raise;
  end;
  Stream.Free;
end;

