procedure AddRichLine(RichEdit: TRichEdit; const StrToAdd: String);
var
   StrLeft: String;
   TempStyle: TFontStyles;
   TempStr: String;
   changed : boolean;

   function FromLeftUntilStr(var OriginalStr: String; const UntilStr: String; const ToEndIfNotFound, Trim: Boolean): String;
   var
      TempPos: Integer;
   begin
      TempPos := Pos(UntilStr, OriginalStr);
      If TempPos > 0 Then
         Begin
            Result := Copy(OriginalStr, 1, TempPos - 1);
            If Trim Then
               Delete(OriginalStr, 1, TempPos - 1);
         End
            Else
         Begin
            If ToEndIfNotFound Then
               Begin
                  Result := OriginalStr;
                  If Trim Then
                     OriginalStr := '';
               End
                  Else
               Result := '';
         End;
   end;

   function FromLeftUntilStrX(var OriginalStr: String; const UntilStr: String; const ToEndIfNotFound, Trim: Boolean): String;
   var
      xStr : String;
   begin
      xStr := Copy(OriginalStr,2,Length(OriginalStr));
      result := '<' + FromLeftUntilStr(xStr,UntilStr,ToEndIfNotFound,Trim);
      OriginalStr := xStr;
   end;

   function StrStartsWith(var OriginalStr: String; const StartsWith: String; const IgnoreCase, Trim: Boolean): Boolean;
   var
      PartOfOriginalStr: String;
      NewStartsWith: String;
   begin
      PartOfOriginalStr := Copy(OriginalStr, 1, Length(StartsWith));
      NewStartsWith := StartsWith;

      If IgnoreCase Then Begin
         PartOfOriginalStr := LowerCase(PartOfOriginalStr);
         NewStartsWith := LowerCase(NewStartsWith);
      End;

      Result := PartOfOriginalStr = NewStartsWith;

      If (Result = True) And (Trim = True) Then
         Delete(OriginalStr, 1, Length(NewStartsWith));
   end;

   procedure AddToStyle(var Style: TFontStyles; AStyle: TFontStyle);
   begin
      If Not (AStyle In Style) Then
         Style := Style + [AStyle];
   end;

   procedure RemoveFromStyle(var Style: TFontStyles; AStyle: TFontStyle);
   begin
      If AStyle In Style Then
         Style := Style - [AStyle];
   end;
begin
   TempStyle := RichEdit.Font.Style;
   StrLeft := StrToAdd;
   RichEdit.SelStart := Length(RichEdit.Text);
   While StrLeft <> '' Do Begin
      If StrStartsWith(StrLeft, '<', True, False) Then
         Begin
            changed := false;
            // Bold Style
            If StrStartsWith(StrLeft, '<b>', True, True) Then
               begin AddToStyle(TempStyle, fsBold); changed := true; end;
            If StrStartsWith(StrLeft, '</b>', True, True) Then
               begin RemoveFromStyle(TempStyle, fsBold); changed := true; end;

            // Italic Style
            If StrStartsWith(StrLeft, '<i>', True, True) Then
               begin AddToStyle(TempStyle, fsItalic); changed := true; end;
            If StrStartsWith(StrLeft, '</i>', True, True) Then
               begin RemoveFromStyle(TempStyle, fsItalic); changed := true; end;

            // Underline Style
            If StrStartsWith(StrLeft, '<u>', True, True) Then
               begin AddToStyle(TempStyle, fsUnderline); changed := true; end;
            If StrStartsWith(StrLeft, '</u>', True, True) Then
               begin RemoveFromStyle(TempStyle, fsUnderline); changed := true; end;

            // Color
            If StrStartsWith(StrLeft, '</color>', True, True) Then
               begin RichEdit.SelAttributes.Color := RichEdit.Font.Color; changed := true; end;
            If StrStartsWith(StrLeft, '<color=', True, True) Then Begin
               TempStr := FromLeftUntilStr(StrLeft, '>', False, True); changed := true;
               Try
                  RichEdit.SelAttributes.Color := StringToColor(TempStr);
               Except
                  RichEdit.SelAttributes.Color := RichEdit.Font.Color;
               End;
               Delete(StrLeft, 1, 1);
            End;

            if not changed then
            begin
                RichEdit.SelAttributes.Style := TempStyle;
                RichEdit.SelText := FromLeftUntilStrX(StrLeft, '<', True, True);
            end;
         End
         Else
         Begin
            RichEdit.SelAttributes.Style := TempStyle;
            RichEdit.SelText := FromLeftUntilStr(StrLeft, '<', True, True);
         End;

      RichEdit.SelStart := Length(RichEdit.Text);
   End;
   RichEdit.SelText := #13#10;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
   AddRichLine(RichEdit1, '<color=clRed><nick> </color><color=clBlack><b>this is the Teks</b></color>. Nice?');
end;
