unit TrCtrls;

Interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

const
  WM_UPDTRANS = WM_USER + 2;
  {this message is called by hook procedure when transparent control should be updated}

type
  TTransObject = Class(TObject)
  private
    FControl: TWinControl;
    FTransparent: Boolean;
    FBackChanged: Boolean;
    ftempdc: THandle;
    ftempbitmap: THandle;
    foldbitmap: THandle;
    procedure KillTempDC;
    procedure SetTransparent(V: Boolean);
    procedure InternalPaint;
    procedure SaveBackGround;
    procedure WMUPDATETRANS;
    procedure WMMove;
  protected
    property Transparent: Boolean read FTransparent write SetTransparent default True;
  public
    Constructor Create(AControl: TWinControl);
    Destructor Destroy; override;
  end;

  TTrTrackBar = Class(TTrackBar)
  private
    FObject: TTransObject;
    function GetTransparent: Boolean;
    procedure SetTransparent(V: Boolean);
  protected
    procedure WMUPDATETRANS(Var Msg: TMessage); message WM_UPDTRANS;
    procedure WMLBUTTONUP(Var Msg: TMessage); message WM_LBUTTONUP;
    procedure WMMove(Var Msg: TMessage); message WM_MOVE;
    procedure WMSize(Var Msg: TMessage); message WM_SIZE;
    procedure WMPAINT(Var Msg: TWMPaint); message WM_PAINT;
    procedure WMEraseBkgnd(Var Msg: TWMEraseBkgnd); message WM_EraseBkgnd;
  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
  published
    property Transparent: Boolean read GetTransparent write SetTransparent;
  end;

  TTrRadioButton = Class(TRadioButton)
  private
    FObject: TTransObject;
    function GetTransparent: Boolean;
    procedure SetTransparent(V: Boolean);
  protected
    procedure WMUPDATETRANS(Var Msg: TMessage); message WM_UPDTRANS;
    procedure BMSETCHECK(Var Msg: TMessage); message BM_SETCHECK;
    procedure WMLBUTTONUP(Var Msg: TMessage); message WM_LBUTTONUP;
    procedure WMMove(Var Msg: TMessage); message WM_MOVE;
    procedure WMSize(Var Msg: TMessage); message WM_SIZE;
    procedure WMPAINT(Var Msg: TWMPaint); message WM_PAINT;
    procedure WMEraseBkgnd(Var Msg: TWMEraseBkgnd); message WM_EraseBkgnd;
  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
  published
    property Transparent: Boolean read GetTransparent write SetTransparent;
  end;

  TCustomTrCheckBox = Class(TCustomCheckBox)
  private
    FObject: TTransObject;
    function GetTransparent: Boolean;
    procedure SetTransparent(V: Boolean);
  protected
    procedure WMUPDATETRANS(Var Msg: TMessage); message WM_UPDTRANS;
    procedure WMMove(Var Msg: TMessage); message WM_MOVE;
    procedure WMSize(Var Msg: TMessage); message WM_SIZE;
    procedure WMPAINT(Var Msg: TWMPaint); message WM_PAINT;
    procedure WMEraseBkgnd(Var Msg: TWMEraseBkgnd); message WM_EraseBkgnd;
  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
  published
    property Transparent: Boolean read GetTransparent write SetTransparent;
  end;

  TCustomTrGroupBox = Class(TCustomGroupBox)
  private
    FObject: TTransObject;
    function GetTransparent: Boolean;
    procedure SetTransparent(V: Boolean);
  protected
    procedure WMUPDATETRANS(Var Msg: TMessage); message WM_UPDTRANS;
    procedure WMMove(Var Msg: TMessage); message WM_MOVE;
    procedure WMSize(Var Msg: TMessage); message WM_SIZE;
    procedure WMPAINT(Var Msg: TWMPaint); message WM_PAINT;
    procedure WMEraseBkgnd(Var Msg: TWMEraseBkgnd); message WM_EraseBkgnd;
  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
  published
  end;

  TTrGroupBox = Class(TCustomTrGroupBox)
  published
    property Align;
    property Caption;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnendDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;


  TTrCheckBox = Class(TCustomTrCheckBox)
  published
    property Transparent;
    property Alignment;
    property AllowGrayed;
    property Caption;
    property Checked;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property State;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnendDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  TCustomTrRadioGroup = Class(TCustomTrGroupBox)
  private
    FButtons: TList;
    FItems: TStrings;
    FItemIndex: Integer;
    FColumns: Integer;
    FReading: Boolean;
    FUpdating: Boolean;
    procedure ArrangeButtons;
    procedure ButtonClick(Sender: TObject);
    procedure ItemsChange(Sender: TObject);
    procedure SetButtonCount(Value: Integer);
    procedure SetColumns(Value: Integer);
    procedure SetItemIndex(Value: Integer);
    procedure SetItems(Value: TStrings);
    procedure UpdateButtons;
    procedure CMEnabledChanged(Var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(Var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize(Var Message: TWMSize); message WM_SIZE;
    procedure WMPaint(Var Message: TWMSize); message WM_paint;
    procedure WMUPDATETRANS(Var Msg: TMessage); message WM_UPDTRANS;
  protected
    procedure ReadState(Reader: TReader); override;
    function CanModify: Boolean; virtual;
    {$IFDEF VER100}
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    {$ELSE}
    procedure GetChildren(Proc: TGetChildProc); override;
    {$ENDIF}
    property Columns: Integer read FColumns write SetColumns default 1;
    property ItemIndex: Integer read FItemIndex write SetItemIndex default - 1;
    property Items: TStrings read FItems write SetItems;
  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
  end;

  TTrRadioGroup = Class(TCustomTrRadioGroup)
  published
    property Align;
    property Caption;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ItemIndex;
    property Items;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnendDrag;
    property OnEnter;
    property OnExit;
    property OnStartDrag;
  end;

procedure Register;

{$IFNDEF VER100}
type
  TWinControlClass = class of TWinControl;
{$ENDIF}

const
  DDF_HALFTONE = $1000;

procedure ControlTransPaintEX(W: TWinControl; BackDC: THandle; Var FTransparent: Boolean;
                                                             X, Y: Integer);
procedure ControlTransPaint(W: TWinControl; BackDC: THandle; Var FTransparent: Boolean);
procedure AddHook(o: TWinControl);
procedure RemoveHook(o: TWinControl);
function GetTransparentColor(dc: THandle; arect: TRect): longint;
procedure TransparentBitBlt(sourcedc, destdc: THandle; arect: TRect; aorigin: TPoint;
                                                       atranscolor: longint);
procedure SaveBackground(A: TWinControl; Var FTempDC, FTempBitmap, FOldBitmap: Thandle);
function Max(A, B: integer): integer;
function Min(A, B: integer): integer;
procedure RegisterTransControl(W: TWinControlClass);
function IsTransControl(W: TWinControl): Boolean;

implementation

const
  FDrawing: Integer = 0;

procedure TCustomTrRadioGroup.WMUPDATETRANS(Var Msg: TMessage);
var
  i: integer;
begin
  Inherited;
  for i := 0 to FButtons.Count - 1 do
  begin
    TWInControl(FButtons[i]).Invalidate;
  end;
end;


procedure TCustomTrRadioGroup.WMPaint(Var Message: TWMSize);
var
  i: integer;
begin
  Inherited;
  for i := 0 to FButtons.Count - 1 do
  begin
    TWInControl(FButtons[i]).Invalidate;
  end;
end;


function GetParentDC(P: TWInControl): THandle;
begin
  Result := 0;
  if P is TTrGroupBox then
    Result := TTrGroupBox(P).FObject.FTempDC
  else
    if P is TTrRadioGroup then
      Result := TTrRadioGroup(P).FObject.FTempDC;
end;


procedure TTransObject.WMUPDATETRANS;
begin
  if FTransparent then
  begin
    fbackchanged := true;
    InternalPaint;
  end;
end;


Constructor TTransObject.Create(AControl: TWinControl);
begin
  Inherited Create;
  FControl := AControl;
  FTransparent := True;
  fBackChanged := true;
  AddHook(AControl);
end;


procedure TTransObject.WMMove;
begin
  if FTransparent then
  begin
    FBackChanged := true;
    InternalPaint;
  end;
end;


procedure TTransObject.KillTempDC;
begin
  if FTempdc <> 0 then
  begin
    SelectObject(ftempdc, foldbitmap);
    DeleteObject(ftempbitmap);
    DeleteDC(ftempdc);
    ftempdc := 0;
  end;
end;


procedure TTransObject.SaveBackGround;
begin
  FBackChanged := false;
  Inc(FDrawing);
  TrCtrls.SaveBackground(FControl, FTempDC, FTempBitmap, FOldBitmap);
  Dec(FDrawing);
end;


procedure TTransObject.InternalPaint;
var
  mParent: TWinControl;
  p: TPoint;
begin
  if (Not FTransparent) or (FDrawing > 0) then
    exit;
  mParent := FControl.Parent;
  while (MParent <> Nil) and (IsTransControl(mParent.Parent)) do
    MParent := MParent.Parent;
  if (MParent <> Nil) and (isTransControl(mParent)) then
  begin
    P.X := 0;
    P.Y := 0;
    P := FControl.ClientToScreen(P);
    P := mparent.ScreenToClient(P);
    ControlTransPaintEX(FControl, GetParentDC(mParent), FTransparent, P.X, P.Y);
  end
  else
  begin
    if fBackChanged then
      SaveBackGround;
    ControlTransPaint(FCOntrol, FTempDC, FTransparent);
  end;
end;


Destructor TTransObject.Destroy;
begin
  RemoveHook(FControl);
  KillTempDC;
  Inherited;
end;


procedure TTransObject.SetTransparent(V: Boolean);
begin
  if V <> FTransparent then
  begin
    if FTransparent then
      RemoveHook(FControl);
    FTransparent := V;
    FBackChanged := True;
    FControl.Invalidate;
    InternalPaint;
    if FTransparent then
      AddHook(FControl);
  end;
end;


procedure TTrRadioButton.BMSETCHECK(Var Msg: TMessage);
begin
  Inherited;
  Invalidate;
end;


procedure TTrRadioButton.WMLBUTTONUP(Var Msg: TMessage);
begin
  Inherited;
  Invalidate;
end;


procedure TTrRadioButton.WMPAINT(Var Msg: TWMPaint);
var
  ps: TPaintStruct;
  R: TRect;
  DC: Thandle;
begin
  with FObject do
  begin
    if FDrawing > 0 then
      exit;
    if not FTransparent then
      Inherited
    else
    begin
      GetUpdateRect(FControl.Handle, R, False);
      if IsRectEmpty(R) then
        exit;
      beginPaint(FControl.handle, ps);
      Msg.result := 0;
      InternalPaint;
      endPaint(FControl.handle, ps);
    end;
  end;
end;


procedure TtrRadioButton.WMEraseBkgnd(Var Msg: TWMEraseBkgnd);
begin
  if FObject.FTransparent then
    Msg.Result := 1
  else
    Inherited;
end;


function TTrRadioButton.GetTransparent: Boolean;
begin
  Result := FObject.Transparent;
end;


procedure TTrRadioButton.SetTransparent(V: Boolean);
begin
  FObject.Transparent := V;
end;


Constructor TTrRadioButton.Create(AOwner: TComponent);
begin
  Inherited;
  FObject := TTransObject.Create(Self);
  ControlStyle := ControlStyle - [csopaque];
end;


Destructor TTrRadioButton.Destroy;
begin
  FObject.Free;
  Inherited;
end;


procedure TTrRadioButton.WMUPDATETRANS(Var Msg: TMessage);
begin
  FObject.WMUPDATETRANS;
end;


procedure TTrRadioButton.WMMove(Var Msg: TMessage);
begin
  Inherited;
  FObject.WMMOVE;
end;


procedure TTrRadioButton.WMSize(Var Msg: TMessage);
begin
  with FObject do
    if FTransparent then
    begin
      KillTempDC;
      Inherited;
      WMMOVE;
    end
    else
      Inherited;
end;


function TCustomTrCheckBox.GetTransparent: Boolean;
begin
  Result := FObject.Transparent;
end;


procedure TCustomTrCheckBox.SetTransparent(V: Boolean);
begin
  FObject.Transparent := V;
end;


Constructor TCustomTrCheckBox.Create(AOwner: TComponent);
begin
  Inherited;
  FObject := TTransObject.Create(Self);
  ControlStyle := ControlStyle - [csopaque];
end;


Destructor TCustomTrCheckBox.Destroy;
begin
  FObject.Free;
  Inherited;
end;


procedure TCustomTrCheckBox.WMUPDATETRANS(Var Msg: TMessage);
begin
  FObject.WMUPDATETRANS;
end;


procedure TCustomTrCheckBox.WMPAINT(Var Msg: TWMPaint);
var
  ps: TPaintStruct;
  R: TRect;
begin
  with FObject do
  begin
    if FDrawing > 0 then
      exit;
    if not FTransparent then
      Inherited
    else
    begin
      GetUpdateRect(FControl.Handle, R, False);
      if IsRectEmpty(R) then
        exit;
      beginPaint(FControl.handle, ps);
      Msg.result := 0;
      InternalPaint;
      endPaint(FControl.handle, ps);
    end;
  end;
end;


procedure TCustomTrCheckBox.WMEraseBkgnd(Var Msg: TWMEraseBkgnd);
begin
  if FObject.FTransparent then
    Msg.Result := 1
  else
    Inherited;
end;


procedure TCustomTrCheckBox.WMMove(Var Msg: TMessage);
begin
  FObject.WMMOVE;
  Inherited;
end;


procedure TCustomTrCheckBox.WMSize(Var Msg: TMessage);
begin
  with FObject do
    if FTransparent then
    begin
      KillTempDC;
      Inherited;
      WMMOVE;
    end
    else
      Inherited;
end;


function TCustomTrGroupBox.GetTransparent: Boolean;
begin
  Result := FObject.Transparent;
end;


procedure TCustomTrGroupBox.SetTransparent(V: Boolean);
begin
  FObject.Transparent := V;
end;


Constructor TCustomTrGroupBox.Create(AOwner: TComponent);
begin
  Inherited;
  FObject := TTransObject.Create(Self);
  ControlStyle := ControlStyle - [csopaque];
end;


Destructor TCustomTrGroupBox.Destroy;
begin
  FObject.Free;
  Inherited;
end;


procedure TCustomTrGroupBox.WMUPDATETRANS(Var Msg: TMessage);
begin
  FObject.WMUPDATETRANS;
end;


procedure TCustomTrGroupBox.WMPAINT(Var Msg: TWMPaint);
var
  ps: TPaintStruct;
  R: TRect;
begin
  with FObject do
  begin
    if FDrawing > 0 then
      exit;
    if not FTransparent then
      Inherited
    else
    begin
      GetUpdateRect(FControl.Handle, R, False);
      if IsRectEmpty(R) then
        exit;
      beginPaint(FControl.handle, ps);
      Msg.result := 0;
      InternalPaint;
      endPaint(FControl.handle, ps);
    end;
  end;
end;


procedure TCustomTrGroupBox.WMEraseBkgnd(Var Msg: TWMEraseBkgnd);
begin
  if FObject.FTransparent then
    Msg.Result := 1
  else
    Inherited;
end;


procedure TCustomTrGroupBox.WMMove(Var Msg: TMessage);
var
  i: integer;
begin
  FObject.WMMOVE;
  Inherited;
  for i := 0 to ComponentCount - 1 do
    if (Components[i] is TWinControl) and (IsTransControl(TWinControl(Components[i]))) then
      PostMessage(TWinControl(Components[i]).Handle, WM_UPDTRANS, 0, 0);
end;


procedure TCustomTrGroupBox.WMSize(Var Msg: TMessage);
begin
  with FObject do
    if FTransparent then
    begin
      KillTempDC;
      Inherited;
      WMMOVE;
    end
    else
      Inherited;
end;


var
  TransClasses: TList;


function IsTransControl(W: TWinControl): Boolean;
var
  i: Integer;
begin
  Result := True;
  For i := 0 to TransClasses.Count - 1 do
    if W is TWinControlClass(TransClasses.Items[i]) then
    begin
      exit;
    end;
  Result := False;
end;


procedure RegisterTransControl(W: TWinControlClass);
begin
  TransClasses.Add(W);
end;


var
  WHook: HHook;
  hooks: TList;

type
  TCWPStruct = Packed record
    lParam: LPARAM;
    wParam: WPARAM;
    message: integer;
    wnd: HWND;
  end;


function CallWndProcHook(nCode: integer; wParam: Longint;
  Var Msg: TCWPStruct): longint; stdcall;
var
  i: integer;
  r: TRect;
  r2: TRect;
  c: TWinControl;

  function IsPaintMsg: boolean;
  begin
    with TWinControl(hooks[i]) do
    begin
      result := false;
      if not HandleAllocated then
        exit;
      if C = Owner then
      begin
        if (msg.message = WM_MOVE) then
          exit;
        Result := True;
        exit;
      end;
      if C.Owner = Owner then
      begin
        GetWindowRect(msg.wnd, r);
        GetWindowRect(handle, r2);
        result := IntersectRect(r, r, r2);
      end;
    end;
  end;

begin
  Result := CallNextHookEx(WHook, nCode, wParam, Longint(@Msg));
  if ((msg.message > CN_BASE) and (msg.message < CN_BASE + 500)) or
      (msg.message = WM_PAINT) or (msg.message = WM_SIZE)
      or (msg.message = WM_MOVE) then
  begin
    c := FindControl(msg.wnd);
    if (c = Nil) or (IsTransControl(c)) then
      exit;
    for i := 0 to hooks.Count - 1 do
    begin
      if (IsPaintMsg) then
        SendMessage(TWinControl(hooks[i]).Handle, WM_UPDTRANS, 0, 0);
    end;
  end;
end;


procedure AddHook(o: TWinControl);
var
  i: integer;
begin
  if hooks.Count = 0 then
    WHook := SetWindowsHookEx(WH_CALLWNDPROC, @CallWndProcHook,
                                                               0, GetCurrentThreadId);
  for i := 0 to Hooks.Count - 1 do
    if Hooks.Items[i] = o then
      exit;
  hooks.Add(o);
end;


procedure RemoveHook(o: TWinControl);
begin
  hooks.Remove(o);
  if hooks.Count = 0 then
    UnHookWindowsHookEx(WHook);
end;


function Min(A, B: integer): integer;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;


function Max(A, B: integer): integer;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;


procedure ControlTransPaint(W: TWinControl; BackDC: THandle; Var FTransparent: Boolean);
begin
  ControlTransPaintEX(W, BackDC, FTransparent, 0, 0);
end;


procedure ControlTransPaintEX(W: TWinControl; BackDC: THandle; Var FTransparent: Boolean;
X, Y: Integer);
var
  DC: THandle;
  memdc: THandle;
  formdc: THandle;
  fbitmap: THandle;
  oldfobject: THandle;
  bitmap: THandle;
  oldmemobject: THandle;
begin
  with W do
  begin
    if ([csReading, csLoading] * ComponentState <> []) or (Parent = Nil)
          or ([csReading, csLoading] * Parent.ComponentState <> [])
          or (Not HandleAllocated) or (Not (visible)) then
      exit;
    dc := GetDC(handle);
    memdc := CreateCompatibleDC(dc);
    formdc := CreateCompatibleDC(dc);
    fbitmap := CreateCompatibleBitmap(dc, width, height);
    oldfobject := SelectObject(formdc, fbitmap);
    bitmap := CreateCompatibleBitmap(dc, width, height);
    oldmemobject := SelectObject(memdc, bitmap);
    BitBlt(formdc, 0, 0, width, height, BackDC, x, y, SRCCOPY);  {1}
    FTransparent := False;
    PaintTo(MemDC, 0, 0);  {2}
    FTransparent := True;
    TransparentBitBlt(MemDC, FormDC, Rect(0, 0, width, height), Point(0, 0),
                                      GetTransparentColor(MemDC, Rect(0, 0, width - 1, height - 1)));  {3}
    BitBlt(dc, 0, 0, width, height, formDC, 0, 0, SRCCOPY);  {4}
    SelectObject(formdc, oldfobject);
    DeleteObject(fbitmap);
    SelectObject(memdc, oldmemobject);
    DeleteObject(bitmap);
    ReleaseDC(handle, dc);
    DeleteDC(memdc);
    DeleteDC(formdc);
  end;
end;


procedure SaveBackground(A: TWinControl; Var FTempDC, FTempBitmap, FOldBitmap: THandle);
var
  dc: THandle;
  formdc: THandle;
  oldfbitmap: THandle;
  fbitmap: THandle;
  fdc: THandle;
begin
  with A do
  begin
    if Parent = Nil then
      exit;
    dc := GetDC(handle);
    fdc := GetDC(parent.handle);
    formdc := CreateCompatibleDC(fdc);
    fbitmap := CreateCompatibleBitmap(fdc, parent.width, parent.height);
    oldfbitmap := SelectObject(formdc, fbitmap);
    if ftempdc = 0 then
    begin
      ftempdc := CreateCompatibleDC(dc);
      ftempbitmap := CreateCompatibleBitmap(dc, width, height);
      foldbitmap := SelectObject(ftempdc, ftempbitmap);
    end;
    IntersectClipRect(formdc, left, top, left + width + 1, top + height +1);
    parent.PaintTo(formdc, 0, 0);
    BitBlt(ftempdc, 0, 0, width, height, formdc, left + 1, top + 1, SRCCOPY);
    SelectObject(formdc, oldfbitmap);
    DeleteObject(fbitmap);
    DeleteDC(formdc);
    ReleaseDC(Parent.Handle, fdc);
    ReleaseDC(handle, dc);
  end;
end;


function GetTransparentColor(dc: THandle; arect: TRect): longint;
begin
  result := GetPixel(dc, arect.left, arect.bottom);
end;


{$IFNDEF VER100}

function TransparentStretchBlt(DstDC: HDC; DstX, DstY, DstW, DstH: Integer;
SrcDC: HDC; SrcX, SrcY, SrcW, SrcH: Integer; MaskDC: HDC; MaskX, MaskY: Integer): Boolean;
const
  ROP_DstCopy = $00AA0029;
var
  MemDC: THandle;
  MemBmp: THandle;
  Save: THandle;
  crText: TColorRef;
  crBack: TColorRef;
  SavePal: HPALETTE;
begin
  Result := True;
  if (Win32Platform = VER_PLATFORM_WIN32_NT) and (SrcW = DstW) and (SrcH = DstH) then
  begin
    MemBmp := CreateCompatibleBitmap(SrcDC, 1, 1);
    MemBmp := SelectObject(MaskDC, MemBmp);
    MaskBlt(DstDC, DstX, DstY, DstW, DstH, SrcDC, SrcX, SrcY, MemBmp,
                     MaskX, MaskY, MakeRop4(ROP_DstCopy, SrcCopy));
    MemBmp := SelectObject(MaskDC, MemBmp);
    DeleteObject(MemBmp);
    exit;
  end;
  SavePal := 0;
  MemDC := CreateCompatibleDC(0);
  MemBmp := CreateCompatibleBitmap(SrcDC, SrcW, SrcH);
  Save := SelectObject(MemDC, MemBmp);
  StretchBlt(MemDC, 0, 0, SrcW, SrcH, MaskDC, MaskX, MaskY, SrcW, SrcH, SrcCopy);
  StretchBlt(MemDC, 0, 0, SrcW, SrcH, SrcDC, SrcX, SrcY, SrcW, SrcH, SrcErase);
  crText := SetTextColor(DstDC, $0);
  crBack := SetBkColor(DstDC, $FFFFFF);
  StretchBlt(DstDC, DstX, DstY, DstW, DstH, MaskDC, MaskX, MaskY, SrcW, SrcH, SrcAnd);
  StretchBlt(DstDC, DstX, DstY, DstW, DstH, MemDC, 0, 0, SrcW, SrcH, SrcInvert);
  SetTextColor(DstDC, crText);
  SetTextColor(DstDC, crBack);
  SelectObject(MemDC, Save);
  DeleteObject(MemBmp);
  DeleteDC(MemDC);
end;

{$ENDIF}

procedure TransparentBitBlt(sourcedc, destdc: THandle; arect: TRect; aorigin: TPoint;
atranscolor: longint);
var
  monobitmap: THandle;
  oldbkcolor: longint;
  monodc: THandle;
  width: integer;
  height: integer;
  oldbitmap: THandle;
begin
  with arect do
  begin
    width := right - left;
    height := bottom - top;
    monodc := CreateCompatibleDC(sourcedc);
    monobitmap := CreateCompatibleBitmap(monodc, width, height);
    try
      oldbitmap := SelectObject(monodc, monobitmap);
      oldbkcolor := SetBkColor(sourcedc, atranscolor);
      BitBlt(monodc, 0, 0, width, height, sourcedc, 0, 0, SRCCOPY);
      SetBkColor(sourcedc, oldbkcolor);
      TransparentStretchBlt(destdc, aorigin.x, aorigin.y, width, height, SourceDC, left,
                                                  top, width, height, monodc, 0, 0);
    finally
      SelectObject(monodc, oldbitmap);
      DeleteDC(monodc);
      DeleteObject(monobitmap);
    end;
  end;
end;


{ TTrGroupButton }

type
  TTrGroupButton = Class(TTrRadioButton)
  private
    FInClick: Boolean;
    Procedure CNCommand(Var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure ChangeScale(M, D: Integer); override;
    procedure KeyDown(Var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(Var Key: Char); override;
  public
    Constructor InternalCreate(RadioGroup: TCustomTrRadioGroup);
    Destructor Destroy; override;
  end;


Constructor TTrGroupButton.InternalCreate(RadioGroup: TCustomTrRadioGroup);
begin
  Inherited Create(RadioGroup);
  RadioGroup.FButtons.Add(Self);
  Visible := False;
  Enabled := RadioGroup.Enabled;
  ParentShowHint := False;
  OnClick := RadioGroup.ButtonClick;
  Parent := RadioGroup;
  RemoveHook(Self);
end;


Destructor TTrGroupButton.Destroy;
begin
  TCustomTrRadioGroup(Owner).FButtons.Remove(Self);
  Inherited Destroy;
end;


procedure TTrGroupButton.CNCommand(Var Message: TWMCommand);
begin
  if not FInClick then
  begin
    FInClick := True;
    try
      if ((Message.NotifyCode = BN_CLICKED) or (Message.NotifyCode = BN_DOUBLECLICKED))
         and TCustomTrRadioGroup(Parent).CanModify then
        Inherited;
    except
      Application.HandleException(Self);
    end;
    FInClick := False;
  end;
end;


procedure TTrGroupButton.ChangeScale(M, D: Integer);
begin
end;


procedure TTrGroupButton.KeyPress(Var Key: Char);
begin
  Inherited KeyPress(Key);
  TCustomTrRadioGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then
  begin
    if not TCustomTrRadioGroup(Parent).CanModify then
      Key := #0;
  end;
end;


procedure TTrGroupButton.KeyDown(Var Key: Word; Shift: TShiftState);
begin
  Inherited KeyDown(Key, Shift);
  TCustomTrRadioGroup(Parent).KeyDown(Key, Shift);
end;


{ TCustomTrRadioGroup }

Constructor TCustomTrRadioGroup.Create(AOwner: TComponent);
begin
  Inherited Create(AOwner);
  ControlStyle := [csSetCaption, csDoubleClicks];
  FButtons := TList.Create;
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChange;
  FItemIndex := -1;
  FColumns := 1;
end;


Destructor TCustomTrRadioGroup.Destroy;
begin
  SetButtonCount(0);
  TStringList(FItems).OnChange := Nil;
  FItems.Free;
  FButtons.Free;
  Inherited Destroy;
end;


procedure TCustomTrRadioGroup.ArrangeButtons;
var
  ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: Integer;
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
  DeferHandle: THandle;
begin
  if (FButtons.Count <> 0) and not FReading then
  begin
    DC := GetDC(0);
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
    ReleaseDC(0, DC);
    ButtonsPerCol := (FButtons.Count + FColumns - 1) div FColumns;
    ButtonWidth := (Width - 10) div FColumns;
    I := Height - Metrics.tmHeight - 5;
    ButtonHeight := I div ButtonsPerCol;
    TopMargin := Metrics.tmHeight + 1 + (I mod ButtonsPerCol) div 2;
    DeferHandle := beginDeferWindowPos(FButtons.Count);
    for I := 0 to FButtons.Count - 1 do
      with TTrGroupButton(FButtons[I]) do
      begin
        DeferHandle := DeferWindowPos(DeferHandle, Handle, 0,
                                     (I div ButtonsPerCol) * ButtonWidth + 8, (I mod ButtonsPerCol) * ButtonHeight
                                     + TopMargin, ButtonWidth, ButtonHeight,
                                     SWP_NOZORDER or SWP_NOACTIVATE);
        Visible := True;
      end;
    endDeferWindowPos(DeferHandle);
  end;
end;


procedure TCustomTrRadioGroup.ButtonClick(Sender: TObject);
begin
  if not FUpdating then
  begin
    FItemIndex := FButtons.IndexOf(Sender);
    {$IFDEF VER100}
    Changed;
    {$ENDIF}
    Click;
  end;
end;


procedure TCustomTrRadioGroup.ItemsChange(Sender: TObject);
begin
  if not FReading then
  begin
    if FItemIndex >= FItems.Count then
      FItemIndex := FItems.Count - 1;
    UpdateButtons;
  end;
end;


procedure TCustomTrRadioGroup.ReadState(Reader: TReader);
begin
  FReading := True;
  Inherited ReadState(Reader);
  FReading := False;
  UpdateButtons;
end;


procedure TCustomTrRadioGroup.SetButtonCount(Value: Integer);
begin
  while FButtons.Count < Value do
    TTrGroupButton.InternalCreate(Self);
  while FButtons.Count > Value do
    TTrGroupButton(FButtons.Last).Free;
end;


procedure TCustomTrRadioGroup.SetColumns(Value: Integer);
begin
  if Value < 1 then
    Value := 1;
  if Value > 16 then
    Value := 16;
  if FColumns <> Value then
  begin
    FColumns := Value;
    ArrangeButtons;
    Invalidate;
  end;
end;


procedure TCustomTrRadioGroup.SetItemIndex(Value: Integer);
begin
  if FReading then
    FItemIndex := Value
  else
  begin
    if Value < - 1 then
      Value := - 1;
    if Value >= FButtons.Count then
      Value := FButtons.Count - 1;
    if FItemIndex <> Value then
    begin
      if FItemIndex >= 0 then
        TTrGroupButton(FButtons[FItemIndex]).Checked := False;
      FItemIndex := Value;
      if FItemIndex >= 0 then
        TTrGroupButton(FButtons[FItemIndex]).Checked := True;
    end;
  end;
end;


procedure TCustomTrRadioGroup.SetItems(Value: TStrings);
begin
  FItems.Assign(Value);
end;


procedure TCustomTrRadioGroup.UpdateButtons;
var
  I: Integer;
begin
  SetButtonCount(FItems.Count);
  for I := 0 to FButtons.Count - 1 do
    TTrGroupButton(FButtons[I]).Caption := FItems[I];
  if FItemIndex >= 0 then
  begin
    FUpdating := True;
    TTrGroupButton(FButtons[FItemIndex]).Checked := True;
    FUpdating := False;
  end;
  ArrangeButtons;
  Invalidate;
end;


procedure TCustomTrRadioGroup.CMEnabledChanged(Var Message: TMessage);
var
  I: Integer;
begin
  Inherited;
  for I := 0 to FButtons.Count - 1 do
    TTrGroupButton(FButtons[I]).Enabled := Enabled;
end;


procedure TCustomTrRadioGroup.CMFontChanged(Var Message: TMessage);
begin
  Inherited;
  ArrangeButtons;
end;


procedure TCustomTrRadioGroup.WMSize(Var Message: TWMSize);
begin
  Inherited;
  ArrangeButtons;
end;


function TCustomTrRadioGroup.CanModify: Boolean;
begin
  Result := True;
end;


{$IFDEF VER100}

procedure TCustomTrRadioGroup.GetChildren
(Proc: TGetChildProc; Root: TComponent);

{$ELSE}

procedure TCustomTrRadioGroup.GetChildren(Proc: TGetChildProc);

{$ENDIF}

begin
end;


{
procedure TTrTrackBar.BMSETCHECK(Var Msg: TMessage);
begin
  Inherited;
  Invalidate;
end;
}


procedure TTrTrackBar.WMLBUTTONUP(Var Msg: TMessage);
begin
  Inherited;
  Invalidate;
end;


procedure TTrTrackBar.WMPAINT(Var Msg: TWMPaint);
var
  ps: TPaintStruct;
  R: TRect;
  DC: Thandle;
begin
  with FObject do
  begin
    if FDrawing > 0 then
      exit;
    if not FTransparent then
      Inherited
    else
    begin
      GetUpdateRect(FControl.Handle, R, False);
      if IsRectEmpty(R) then
        exit;
      beginPaint(FControl.handle, ps);
      Msg.result := 0;
      InternalPaint;
      endPaint(FControl.handle, ps);
    end;
  end;
end;


procedure TTrTrackBar.WMEraseBkgnd(Var Msg: TWMEraseBkgnd);
begin
  if FObject.FTransparent then
    Msg.Result := 1
  else
    Inherited;
end;


function TTrTrackBar.GetTransparent: Boolean;
begin
  Result := FObject.Transparent;
end;


procedure TTrTrackBar.SetTransparent(V: Boolean);
begin
  FObject.Transparent := V;
end;


Constructor TTrTrackBar.Create(AOwner: TComponent);
begin
  Inherited;
  FObject := TTransObject.Create(Self);
  ControlStyle := ControlStyle - [csopaque];
end;


Destructor TTrTrackBar.Destroy;
begin
  FObject.Free;
  Inherited;
end;


procedure TTrTrackBar.WMUPDATETRANS(Var Msg: TMessage);
begin
  FObject.WMUPDATETRANS;
end;


procedure TTrTrackBar.WMMove(Var Msg: TMessage);
begin
  Inherited;
  FObject.WMMOVE;
end;


procedure TTrTrackBar.WMSize(Var Msg: TMessage);
begin
  With FObject do
    if FTransparent then
    begin
      KillTempDC;
      Inherited;
      WMMOVE;
    end
    else
      Inherited;
end;


procedure Register;
begin
  RegisterComponents('Trans Controls',  [TTrGroupBox, TTrRadioButton, TTrCheckBox,
                                            TTrRadioGroup, TTrTrackBar]);
end;


Initialization
  hooks := TList.Create;
  TransClasses := TList.Create;
  RegisterTransControl(TTrRadioButton);
  RegisterTransControl(TTrCheckBox);
  RegisterTransControl(TTrGroupBox);
  RegisterTransControl(TTrRadioGroup);
  RegisterTransControl(TTrGroupButton);
  RegisterTransControl(TTrTrackBar);

Finalization
  if hooks.Count > 0 then
    UnHookWindowsHookEx(WHook);
  hooks.Free;
  TRansClasses.Free;

end.
