(*This code creates a tray icon which when right-clicked, pops up a menu, and 
when double-clicked, displays the main form. You simply need to create a new 
Delphi project, drop a PopupMenu on the form and copy the event handler 
code, copy the WndProc procedure and declaration code (including the 
"procedure WndProc... 
Begin 
... 
End;") 
and copy the FormCreate code. You will have a system tray icon with your 
app's icon, and its title as a tooltip.The form will not have a button on 
the taskbar. 
This code should work with D3, it works with D2 standard. 
*) 
unit MainForm; 

interface 

uses 
  Windows, ShellAPI, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  Dialogs, 
  Menus; 

type 
  TForm1 = class(TForm) 
    PopupMenu1: TPopupMenu; 
    ShowMainForm1: TMenuItem; 
    ExitApplication1: TMenuItem; 
    procedure FormCreate(Sender: TObject); 
    procedure ShowMainForm1Click(Sender: TObject); 
    procedure FormClose(Sender: TObject; var Action: TCloseAction); 
    procedure ExitApplication1Click(Sender: TObject); 
  private 
    { Private declarations } 
    procedure WndProc(var Msg: TMessage); override; 
  public 
    { Public declarations } 
    IconNotifyData: TNotifyIconData; 
  end; 
var 
  Form1: TForm1; 

implementation 
{$R *.DFM} 

procedure TForm1.FormCreate(Sender: TObject); 
begin 
  showwindow(Application.Handle, SW_HIDE); {Comment out if you want a taskbar 
button} 
  //Now set up the IconNotifyData structure so that it receives 
  //the window messages sent to the application and displays 
  //the application's tips 
  with IconNotifyData do 
  begin 
    hIcon := Application.Icon.Handle; 
    uCallbackMessage := WM_USER + 1; 
    cbSize := sizeof(IconNotifyData); 
    Wnd := Handle; 
    uID := 100; 
    uFlags := NIF_MESSAGE + NIF_ICON + NIF_TIP; 
  end; 
  //Copy the Application's Title into the tooltip for the icon 
  StrPCopy(IconNotifyData.szTip, Application.Title); 
  //Add the Icon to the system tray and use the 
  //the structure and its values 
  Shell_NotifyIcon(NIM_ADD, @IconNotifyData); 
end; 

procedure TForm1.WndProc(var Msg: TMessage); 
var 
  p: TPoint; 
begin 
  case Msg.Msg of 
    WM_USER + 1: 
      case Msg.lParam of 
        WM_RBUTTONDOWN: 
        begin 
          GetCursorPos(p); 
          PopupMenu1.Popup(p.x, p.y); 
        end; 
        WM_LBUTTONDBLCLK: Form1.Show; 
      end; 
  end; 
  inherited; 
end; 

procedure TForm1.ShowMainForm1Click(Sender: TObject); 
begin 
  Form1.Show; 
end; 

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction); 
begin 
  //when the user clicks the close button in the corner, 
  //only hide the form, not exit the app. 
  Action := caNone; 
  Form1.Hide; 
end; 

procedure TForm1.ExitApplication1Click(Sender: TObject); 
begin 
  Shell_NotifyIcon(NIM_DELETE, @IconNotifyData); 
  Application.ProcessMessages; 
  Application.Terminate; 
end; 

end.
